@@ -18,6 +18,8 @@ from __future__ import annotations
 
 import pytest
 
+from airflow.providers.standard.exceptions import HITLRejectException, HITLTimeoutError, HITLTriggerEventError
+
 from tests_common.test_utils.version_compat import AIRFLOW_V_3_1_PLUS
 
 if not AIRFLOW_V_3_1_PLUS:
@@ -43,12 +45,16 @@ from airflow.providers.standard.operators.hitl import (
 )
 from airflow.sdk import Param, timezone
 from airflow.sdk.definitions.param import ParamsDict
+from airflow.sdk.execution_time.hitl import HITLUser
+from airflow.utils.context import Context
 
 from tests_common.test_utils.config import conf_vars
 
 if TYPE_CHECKING:
     from sqlalchemy.orm import Session
 
+    from airflow.sdk.definitions.context import Context
+
     from tests_common.pytest_plugin import DagMaker
 
 pytestmark = pytest.mark.db_test
@@ -66,7 +72,7 @@ def hitl_task_and_ti_for_generating_link(dag_maker: DagMaker) -> tuple[HITLOpera
             options=["1", "2", "3", "4", "5"],
             body="This is body",
             defaults=["1"],
-            respondents="test",
+            assigned_users=HITLUser(id="test", name="test"),
             multiple=True,
             params=ParamsDict({"input_1": 1, "input_2": 2, "input_3": 3}),
         )
@@ -74,6 +80,37 @@ def hitl_task_and_ti_for_generating_link(dag_maker: DagMaker) -> tuple[HITLOpera
     return task, dag_maker.run_ti(task.task_id, dr)
 
 
+@pytest.fixture
+def get_context_from_model_ti(mock_supervisor_comms):
+    def _get_context(ti: TaskInstance) -> Context:
+        from airflow.api_fastapi.execution_api.datamodels.taskinstance import (
+            DagRun as DRDataModel,
+            TaskInstance as TIDataModel,
+            TIRunContext,
+        )
+        from airflow.sdk.execution_time.task_runner import RuntimeTaskInstance
+
+        # make mypy happy
+        assert ti is not None
+
+        dag_run = ti.dag_run
+        ti_model = TIDataModel.model_validate(ti, from_attributes=True)
+        runtime_ti = RuntimeTaskInstance.model_construct(
+            **ti_model.model_dump(exclude_unset=True),
+            task=ti.task,
+            _ti_context_from_server=TIRunContext(
+                dag_run=DRDataModel.model_validate(dag_run, from_attributes=True),
+                max_tries=ti.max_tries,
+                variables=[],
+                connections=[],
+                xcom_keys_to_clear=[],
+            ),
+        )
+        return runtime_ti.get_template_context()
+
+    return _get_context
+
+
 class TestHITLOperator:
     def test_validate_options(self) -> None:
         hitl_op = HITLOperator(
@@ -165,7 +202,7 @@ class TestHITLOperator:
                 options=["1", "2", "3", "4", "5"],
                 body="This is body",
                 defaults=["1"],
-                respondents="test",
+                assigned_users=HITLUser(id="test", name="test"),
                 multiple=False,
                 params=ParamsDict({"input_1": 1}),
                 notifiers=[notifier],
@@ -181,10 +218,9 @@ class TestHITLOperator:
         assert hitl_detail_model.defaults == ["1"]
         assert hitl_detail_model.multiple is False
         assert hitl_detail_model.params == {"input_1": 1}
-        assert hitl_detail_model.respondents == ["test"]
-        assert hitl_detail_model.response_at is None
-        assert hitl_detail_model.responded_user_id is None
-        assert hitl_detail_model.responded_user_name is None
+        assert hitl_detail_model.assignees == [{"id": "test", "name": "test"}]
+        assert hitl_detail_model.responded_at is None
+        assert hitl_detail_model.responded_by is None
         assert hitl_detail_model.chosen_options is None
         assert hitl_detail_model.params_input == {}
 
@@ -230,13 +266,46 @@ class TestHITLOperator:
             params={"input": 1},
         )
 
+        responded_at_dt = timezone.utcnow()
+
         ret = hitl_op.execute_complete(
             context={},
-            event={"chosen_options": ["1"], "params_input": {"input": 2}},
+            event={
+                "chosen_options": ["1"],
+                "params_input": {"input": 2},
+                "responded_at": responded_at_dt,
+                "responded_by_user": {"id": "test", "name": "test"},
+            },
         )
 
-        assert ret["chosen_options"] == ["1"]
-        assert ret["params_input"] == {"input": 2}
+        assert ret == {
+            "chosen_options": ["1"],
+            "params_input": {"input": 2},
+            "responded_at": responded_at_dt,
+            "responded_by_user": {"id": "test", "name": "test"},
+        }
+
+    @pytest.mark.parametrize(
+        "event, expected_exception",
+        [
+            ({"error": "unknown", "error_type": "unknown"}, HITLTriggerEventError),
+            ({"error": "this is timeotu", "error_type": "timeout"}, HITLTimeoutError),
+        ],
+    )
+    def test_process_trigger_event_error(
+        self,
+        event: dict[str, Any],
+        expected_exception,
+    ) -> None:
+        hitl_op = HITLOperator(
+            task_id="hitl_test",
+            subject="This is subject",
+            body="This is body",
+            options=["1", "2", "3", "4", "5"],
+            params={"input": 1},
+        )
+        with pytest.raises(expected_exception):
+            hitl_op.process_trigger_event_error(event)
 
     def test_validate_chosen_options_with_invalid_content(self) -> None:
         hitl_op = HITLOperator(
@@ -253,6 +322,7 @@ class TestHITLOperator:
                 event={
                     "chosen_options": ["not exists"],
                     "params_input": {"input": 2},
+                    "responded_by_user": {"id": "test", "name": "test"},
                 },
             )
 
@@ -271,6 +341,7 @@ class TestHITLOperator:
                 event={
                     "chosen_options": ["1"],
                     "params_input": {"no such key": 2, "input": 333},
+                    "responded_by_user": {"id": "test", "name": "test"},
                 },
             )
 
@@ -395,17 +466,28 @@ class TestApprovalOperator:
             subject="This is subject",
         )
 
+        responded_at_dt = timezone.utcnow()
+
         ret = hitl_op.execute_complete(
             context={},
-            event={"chosen_options": ["Approve"], "params_input": {}},
+            event={
+                "chosen_options": ["Approve"],
+                "params_input": {},
+                "responded_at": responded_at_dt,
+                "responded_by_user": {"id": "test", "name": "test"},
+            },
         )
 
         assert ret == {
             "chosen_options": ["Approve"],
             "params_input": {},
+            "responded_at": responded_at_dt,
+            "responded_by_user": {"id": "test", "name": "test"},
         }
 
-    def test_execute_complete_with_downstream_tasks(self, dag_maker) -> None:
+    def test_execute_complete_with_downstream_tasks(
+        self, dag_maker: DagMaker, get_context_from_model_ti
+    ) -> None:
         with dag_maker("hitl_test_dag", serialized=True):
             hitl_op = ApprovalOperator(
                 task_id="hitl_test",
@@ -415,14 +497,38 @@ class TestApprovalOperator:
 
         dr = dag_maker.create_dagrun()
         ti = dr.get_task_instance("hitl_test")
-
         with pytest.raises(DownstreamTasksSkipped) as exc_info:
             hitl_op.execute_complete(
-                context={"ti": ti, "task": ti.task},
-                event={"chosen_options": ["Reject"], "params_input": {}},
+                context=get_context_from_model_ti(ti),
+                event={
+                    "chosen_options": ["Reject"],
+                    "params_input": {},
+                    "responded_at": timezone.utcnow(),
+                    "responded_by_user": {"id": "test", "name": "test"},
+                },
             )
         assert set(exc_info.value.tasks) == {"op1"}
 
+    def test_execute_complete_with_fail_on_reject_set_to_true(
+        self, dag_maker: DagMaker, get_context_from_model_ti
+    ) -> None:
+        with dag_maker("hitl_test_dag", serialized=True):
+            hitl_op = ApprovalOperator(task_id="hitl_test", subject="This is subject", fail_on_reject=True)
+            (hitl_op >> EmptyOperator(task_id="op1"))
+
+        dr = dag_maker.create_dagrun()
+        ti = dr.get_task_instance("hitl_test")
+        with pytest.raises(HITLRejectException):
+            hitl_op.execute_complete(
+                context=get_context_from_model_ti(ti),
+                event={
+                    "chosen_options": ["Reject"],
+                    "params_input": {},
+                    "responded_at": timezone.utcnow(),
+                    "responded_by_user": {"id": "test", "name": "test"},
+                },
+            )
+
 
 class TestHITLEntryOperator:
     def test_init_without_options_and_default(self) -> None:
@@ -462,7 +568,7 @@ class TestHITLEntryOperator:
 
 
 class TestHITLBranchOperator:
-    def test_execute_complete(self, dag_maker) -> None:
+    def test_execute_complete(self, dag_maker: DagMaker, get_context_from_model_ti) -> None:
         with dag_maker("hitl_test_dag", serialized=True):
             branch_op = HITLBranchOperator(
                 task_id="make_choice",
@@ -476,15 +582,19 @@ class TestHITLBranchOperator:
         ti = dr.get_task_instance("make_choice")
         with pytest.raises(DownstreamTasksSkipped) as exc_info:
             branch_op.execute_complete(
-                context={"ti": ti, "task": ti.task},
+                context=get_context_from_model_ti(ti),
                 event={
                     "chosen_options": ["branch_1"],
                     "params_input": {},
+                    "responded_at": timezone.utcnow(),
+                    "responded_by_user": {"id": "test", "name": "test"},
                 },
             )
         assert set(exc_info.value.tasks) == set((f"branch_{i}", -1) for i in range(2, 6))
 
-    def test_execute_complete_with_multiple_branches(self, dag_maker) -> None:
+    def test_execute_complete_with_multiple_branches(
+        self, dag_maker: DagMaker, get_context_from_model_ti
+    ) -> None:
         with dag_maker("hitl_test_dag", serialized=True):
             branch_op = HITLBranchOperator(
                 task_id="make_choice",
@@ -495,19 +605,23 @@ class TestHITLBranchOperator:
 
             branch_op >> [EmptyOperator(task_id=f"branch_{i}") for i in range(1, 6)]
 
+        responded_at_dt = timezone.utcnow()
+
         dr = dag_maker.create_dagrun()
         ti = dr.get_task_instance("make_choice")
         with pytest.raises(DownstreamTasksSkipped) as exc_info:
             branch_op.execute_complete(
-                context={"ti": ti, "task": ti.task},
+                context=get_context_from_model_ti(ti),
                 event={
                     "chosen_options": [f"branch_{i}" for i in range(1, 4)],
                     "params_input": {},
+                    "responded_at": responded_at_dt,
+                    "responded_by_user": {"id": "test", "name": "test"},
                 },
             )
         assert set(exc_info.value.tasks) == set((f"branch_{i}", -1) for i in range(4, 6))
 
-    def test_mapping_applies_for_single_choice(self, dag_maker):
+    def test_mapping_applies_for_single_choice(self, dag_maker: DagMaker, get_context_from_model_ti) -> None:
         # ["Approve"]; map -> "publish"
         with dag_maker("hitl_map_dag", serialized=True):
             op = HITLBranchOperator(
@@ -523,13 +637,18 @@ class TestHITLBranchOperator:
 
         with pytest.raises(DownstreamTasksSkipped) as exc:
             op.execute_complete(
-                context={"ti": ti, "task": ti.task},
-                event={"chosen_options": ["Approve"], "params_input": {}},
+                context=get_context_from_model_ti(ti),
+                event={
+                    "chosen_options": ["Approve"],
+                    "params_input": {},
+                    "responded_at": timezone.utcnow(),
+                    "responded_by_user": {"id": "test", "name": "test"},
+                },
             )
         # checks to see that the "archive" task was skipped
         assert set(exc.value.tasks) == {("archive", -1)}
 
-    def test_mapping_with_multiple_choices(self, dag_maker):
+    def test_mapping_with_multiple_choices(self, dag_maker: DagMaker, get_context_from_model_ti) -> None:
         # multiple=True; mapping applied per option; no dedup implied
         with dag_maker("hitl_map_dag", serialized=True):
             op = HITLBranchOperator(
@@ -550,13 +669,18 @@ class TestHITLBranchOperator:
 
         with pytest.raises(DownstreamTasksSkipped) as exc:
             op.execute_complete(
-                context={"ti": ti, "task": ti.task},
-                event={"chosen_options": ["Approve", "KeepAsIs"], "params_input": {}},
+                context=get_context_from_model_ti(ti),
+                event={
+                    "chosen_options": ["Approve", "KeepAsIs"],
+                    "params_input": {},
+                    "responded_at": timezone.utcnow(),
+                    "responded_by_user": {"id": "test", "name": "test"},
+                },
             )
         # publish + keep chosen → only "other" skipped
         assert set(exc.value.tasks) == {("other", -1)}
 
-    def test_fallback_to_option_when_not_mapped(self, dag_maker):
+    def test_fallback_to_option_when_not_mapped(self, dag_maker: DagMaker, get_context_from_model_ti) -> None:
         # No mapping: option must match downstream task_id
         with dag_maker("hitl_map_dag", serialized=True):
             op = HITLBranchOperator(
@@ -571,12 +695,19 @@ class TestHITLBranchOperator:
 
         with pytest.raises(DownstreamTasksSkipped) as exc:
             op.execute_complete(
-                context={"ti": ti, "task": ti.task},
-                event={"chosen_options": ["branch_2"], "params_input": {}},
+                context=get_context_from_model_ti(ti),
+                event={
+                    "chosen_options": ["branch_2"],
+                    "params_input": {},
+                    "responded_at": timezone.utcnow(),
+                    "responded_by_user": {"id": "test", "name": "test"},
+                },
             )
         assert set(exc.value.tasks) == {("branch_1", -1)}
 
-    def test_error_if_mapped_branch_not_direct_downstream(self, dag_maker):
+    def test_error_if_mapped_branch_not_direct_downstream(
+        self, dag_maker: DagMaker, get_context_from_model_ti
+    ):
         # Don't add the mapped task downstream → expect a clean error
         with dag_maker("hitl_map_dag", serialized=True):
             op = HITLBranchOperator(
@@ -592,8 +723,13 @@ class TestHITLBranchOperator:
 
         with pytest.raises(AirflowException, match="downstream|not found"):
             op.execute_complete(
-                context={"ti": ti, "task": ti.task},
-                event={"chosen_options": ["Approve"], "params_input": {}},
+                context=get_context_from_model_ti(ti),
+                event={
+                    "chosen_options": ["Approve"],
+                    "params_input": {},
+                    "responded_at": timezone.utcnow(),
+                    "responded_by_user": {"id": "test", "name": "test"},
+                },
             )
 
     @pytest.mark.parametrize("bad", [123, ["publish"], {"x": "y"}, b"publish"])
